@@ -7,11 +7,12 @@
  * Copyright (c) 1987-1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  * Copyright (c) 1993-1996 Lucent Technologies.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tcl.h 1.24 98/08/06 12:10:41
+ * RCS: @(#) $Id: tcl.h,v 1.29 1998/10/17 00:19:24 escoffon Exp $
  */
 
 #ifndef _TCL
@@ -26,6 +27,8 @@
  * unix/configure.in
  * win/makefile.bc	(only if major.minor changes, not patchlevel)
  * win/makefile.vc	(only if major.minor changes, not patchlevel)
+ * win/README
+ * win/README.binary
  *
  * The release level should be  0 for alpha, 1 for beta, and 2 for
  * final/patch.  The release serial value is the number that follows the
@@ -38,10 +41,10 @@
 #define TCL_MAJOR_VERSION   8
 #define TCL_MINOR_VERSION   0
 #define TCL_RELEASE_LEVEL   2
-#define TCL_RELEASE_SERIAL  3
+#define TCL_RELEASE_SERIAL  4
 
 #define TCL_VERSION	    "8.0"
-#define TCL_PATCH_LEVEL	    "8.0.3+"
+#define TCL_PATCH_LEVEL	    "8.0.4"
 
 /*
  * The following definitions set up the proper options for Windows
@@ -506,7 +509,7 @@ EXTERN Tcl_Obj *	Tcl_NewListObj _ANSI_ARGS_((int objc,
 			    Tcl_Obj *CONST objv[]));
 EXTERN Tcl_Obj *	Tcl_NewLongObj _ANSI_ARGS_((long longValue));
 EXTERN Tcl_Obj *	Tcl_NewObj _ANSI_ARGS_((void));
-EXTERN Tcl_Obj *	Tcl_NewStringObj _ANSI_ARGS_((CONST char *bytes,
+EXTERN Tcl_Obj *	Tcl_NewStringObj _ANSI_ARGS_((char *bytes,
 			    int length));
 
 #ifdef TCL_MEM_DEBUG
@@ -654,14 +657,6 @@ typedef struct Tcl_DString {
 #define TCL_MAX_PREC 17
 #define TCL_DOUBLE_SPACE (TCL_MAX_PREC+10)
 
-/*
- * Definition for a number of bytes of buffer space sufficient to hold the
- * string representation of an integer in base 10 (assuming the existence
- * of 64-bit integers).
- */
-
-#define TCL_INTEGER_SPACE	24
-
 /*
  * Flag that may be passed to Tcl_ConvertElement to force it not to
  * output braces (careful!  if you change this flag be sure to change
@@ -731,10 +726,6 @@ EXTERN char *		Tcl_Alloc _ANSI_ARGS_((unsigned int size));
 EXTERN void		Tcl_Free _ANSI_ARGS_((char *ptr));
 EXTERN char *		Tcl_Realloc _ANSI_ARGS_((char *ptr,
 			    unsigned int size));
-EXTERN void		Tcl_InitMemory _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN int		Tcl_DumpActiveMemory _ANSI_ARGS_((char *fileName));
-EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
-			    int line));
 
 #ifdef TCL_MEM_DEBUG
 
@@ -745,6 +736,10 @@ EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
 #  define ckfree(x)  Tcl_DbCkfree(x, __FILE__, __LINE__)
 #  define ckrealloc(x,y) Tcl_DbCkrealloc((x), (y),__FILE__, __LINE__)
 
+EXTERN int		Tcl_DumpActiveMemory _ANSI_ARGS_((char *fileName));
+EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
+			    int line));
+
 #else
 
 #  if USE_TCLALLOC
@@ -756,7 +751,6 @@ EXTERN void		Tcl_ValidateAllMemory _ANSI_ARGS_((char *file,
 #     define ckfree(x)  free(x)
 #     define ckrealloc(x,y) realloc(x,y)
 #  endif
-#  define Tcl_InitMemory(x)
 #  define Tcl_DumpActiveMemory(x)
 #  define Tcl_ValidateAllMemory(x,y)
 
@@ -940,12 +934,6 @@ typedef struct Tcl_Time {
 #define TCL_READABLE	(1<<1)
 #define TCL_WRITABLE	(1<<2)
 #define TCL_EXCEPTION	(1<<3)
-#define TCL_ACTIVE	0
-
-typedef struct Tcl_StaticFile {
-    CONST char *name;
-    char **content;
-} Tcl_StaticFile;
 
 /*
  * Flag values to pass to Tcl_OpenCommandChannel to indicate the
@@ -995,15 +983,6 @@ typedef enum Tcl_EolTranslation {
     TCL_TRANSLATE_CRLF			/* Eol == \r\n. */
 } Tcl_EolTranslation;
 
-/*
- * Definitions for different byteorders.
- */
-
-typedef int Tcl_ByteOrder;
-
-#define TCL_BIGENDIAN 0
-#define TCL_SMALLENDIAN 1
-
 /*
  * struct Tcl_ChannelType:
  *
@@ -1064,21 +1043,21 @@ typedef enum Tcl_PathType {
  */
 
 EXTERN void		Tcl_AddErrorInfo _ANSI_ARGS_((Tcl_Interp *interp,
-			    CONST char *message));
+			    char *message));
 EXTERN void		Tcl_AddObjErrorInfo _ANSI_ARGS_((Tcl_Interp *interp,
-			    CONST char *message, int length));
+			    char *message, int length));
 EXTERN void		Tcl_AllowExceptions _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		Tcl_AppendAllObjTypes _ANSI_ARGS_((
 			    Tcl_Interp *interp, Tcl_Obj *objPtr));
 EXTERN void		Tcl_AppendElement _ANSI_ARGS_((Tcl_Interp *interp,
-			    CONST char *string));
+			    char *string));
 EXTERN void		Tcl_AppendResult _ANSI_ARGS_(
 			    TCL_VARARGS(Tcl_Interp *,interp));
 EXTERN void		Tcl_AppendToObj _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    char *bytes, int length));
 EXTERN void		Tcl_AppendStringsToObj _ANSI_ARGS_(
 			    TCL_VARARGS(Tcl_Obj *,interp));
-extern int		Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN Tcl_AsyncHandler	Tcl_AsyncCreate _ANSI_ARGS_((Tcl_AsyncProc *proc,
 			    ClientData clientData));
 EXTERN void		Tcl_AsyncDelete _ANSI_ARGS_((Tcl_AsyncHandler async));
@@ -1176,10 +1155,8 @@ EXTERN Tcl_Obj *	Tcl_DbNewListObj _ANSI_ARGS_((int objc,
 EXTERN Tcl_Obj *	Tcl_DbNewLongObj _ANSI_ARGS_((long longValue,
                             char *file, int line));
 EXTERN Tcl_Obj *	Tcl_DbNewObj _ANSI_ARGS_((char *file, int line));
-EXTERN Tcl_Obj *	Tcl_DbNewStringObj _ANSI_ARGS_((CONST char *bytes,
+EXTERN Tcl_Obj *	Tcl_DbNewStringObj _ANSI_ARGS_((char *bytes,
 			    int length, char *file, int line));
-EXTERN void		Tcl_DefineStaticFile _ANSI_ARGS_((
-			    Tcl_StaticFile *table));
 EXTERN void		Tcl_DeleteAssocData _ANSI_ARGS_((Tcl_Interp *interp,
                             char *name));
 EXTERN int		Tcl_DeleteCommand _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1265,9 +1242,7 @@ EXTERN int		Tcl_ExprObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN int		Tcl_ExprString _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string));
 EXTERN void		Tcl_Finalize _ANSI_ARGS_((void));
-EXTERN void		Tcl_FindExecutable _ANSI_ARGS_((CONST char *argv0));
-EXTERN VOID *		Tcl_FindPackageHandle _ANSI_ARGS_((
-			    CONST char *packageName));
+EXTERN void		Tcl_FindExecutable _ANSI_ARGS_((char *argv0));
 EXTERN Tcl_HashEntry *	Tcl_FirstHashEntry _ANSI_ARGS_((
 			    Tcl_HashTable *tablePtr,
 			    Tcl_HashSearch *searchPtr));
@@ -1293,12 +1268,6 @@ EXTERN Tcl_Channel	Tcl_GetChannel _ANSI_ARGS_((Tcl_Interp *interp,
 	        	    char *chanName, int *modePtr));
 EXTERN int		Tcl_GetChannelBufferSize _ANSI_ARGS_((
     			    Tcl_Channel chan));
-  /* Andreas Kupries <a.kupries@westend.com>, 05/31/1997.
-   * Support of Tcl-Trf (binio).
-   */
-EXTERN int		Tcl_GetChannelByteorder _ANSI_ARGS_((
-			    Tcl_Channel chan));
-EXTERN int		Tcl_GetHostByteorder _ANSI_ARGS_((void));
 EXTERN int		Tcl_GetChannelHandle _ANSI_ARGS_((Tcl_Channel chan,
 	        	    int direction, ClientData *handlePtr));
 EXTERN ClientData	Tcl_GetChannelInstanceData _ANSI_ARGS_((
@@ -1313,14 +1282,12 @@ EXTERN int		Tcl_GetCommandInfo _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *cmdName, Tcl_CmdInfo *infoPtr));
 EXTERN char *		Tcl_GetCommandName _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Command command));
-EXTERN char *		Tcl_GetCwd _ANSI_ARGS_((char *buf, int len));
 EXTERN int		Tcl_GetDouble _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, double *doublePtr));
 EXTERN int		Tcl_GetDoubleFromObj _ANSI_ARGS_((
 			    Tcl_Interp *interp, Tcl_Obj *objPtr,
 			    double *doublePtr));
 EXTERN int		Tcl_GetErrno _ANSI_ARGS_((void));
-EXTERN int		Tcl_GetErrorLine _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN char *		Tcl_GetHostName _ANSI_ARGS_((void));
 EXTERN int		Tcl_GetIndexFromObj _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *objPtr, char **tablePtr, char *msg,
@@ -1336,14 +1303,10 @@ EXTERN int		Tcl_GetLongFromObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN Tcl_Interp *	Tcl_GetMaster _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN CONST char *	Tcl_GetNameOfExecutable _ANSI_ARGS_((void));
 EXTERN Tcl_Obj *	Tcl_GetObjResult _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN Tcl_Obj *	Tcl_GetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *part1, char *part2, int flags));
 EXTERN Tcl_ObjType *	Tcl_GetObjType _ANSI_ARGS_((char *typeName));
 EXTERN int		Tcl_GetOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int write, int checkUsage,
 			    ClientData *filePtr));
-EXTERN Tcl_Command	Tcl_GetOriginalCommand _ANSI_ARGS_((
-			    Tcl_Command command));
 EXTERN Tcl_PathType	Tcl_GetPathType _ANSI_ARGS_((char *path));
 EXTERN int		Tcl_Gets _ANSI_ARGS_((Tcl_Channel chan,
         		    Tcl_DString *dsPtr));
@@ -1352,9 +1315,7 @@ EXTERN int		Tcl_GetsObj _ANSI_ARGS_((Tcl_Channel chan,
 EXTERN int		Tcl_GetServiceMode _ANSI_ARGS_((void));
 EXTERN Tcl_Interp *	Tcl_GetSlave _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *slaveName));
-EXTERN char **		Tcl_GetStaticFile _ANSI_ARGS_((CONST char *rsrcName));
 EXTERN Tcl_Channel	Tcl_GetStdChannel _ANSI_ARGS_((int type));
-EXTERN char *		Tcl_GetString _ANSI_ARGS_((Tcl_Obj *objPtr));
 EXTERN char *		Tcl_GetStringFromObj _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    int *lengthPtr));
 EXTERN char *		Tcl_GetStringResult _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1370,10 +1331,9 @@ EXTERN char *		Tcl_HashStats _ANSI_ARGS_((Tcl_HashTable *tablePtr));
 EXTERN int		Tcl_HideCommand _ANSI_ARGS_((Tcl_Interp *interp,
 		            char *cmdName, char *hiddenCmdToken));
 EXTERN int		Tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN int		Tcl_InitStandAlone _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN void		Tcl_InitHashTable _ANSI_ARGS_((Tcl_HashTable *tablePtr,
 			    int keyType));
-EXTERN ClientData	Tcl_InitNotifier _ANSI_ARGS_((void));
+EXTERN void		Tcl_InitMemory _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		Tcl_InputBlocked _ANSI_ARGS_((Tcl_Channel chan));
 EXTERN int		Tcl_InputBuffered _ANSI_ARGS_((Tcl_Channel chan));
 EXTERN int		Tcl_InterpDeleted _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1403,7 +1363,6 @@ EXTERN int		Tcl_ListObjReplace _ANSI_ARGS_((Tcl_Interp *interp,
 			    int objc, Tcl_Obj *CONST objv[]));
 EXTERN void		Tcl_Main _ANSI_ARGS_((int argc, char **argv,
 			    Tcl_AppInitProc *appInitProc));
-EXTERN void		Tcl_MainLoop _ANSI_ARGS_((void));
 EXTERN Tcl_Channel	Tcl_MakeFileChannel _ANSI_ARGS_((ClientData handle,
 			    int mode));
 EXTERN int		Tcl_MakeSafe _ANSI_ARGS_((Tcl_Interp *interp));
@@ -1456,8 +1415,7 @@ EXTERN int		Tcl_RecordAndEvalObj _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN Tcl_RegExp	Tcl_RegExpCompile _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string));
 EXTERN int		Tcl_RegExpExec _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tcl_RegExp regexp, CONST char *string,
-			    CONST char *start));
+			    Tcl_RegExp regexp, char *string, char *start));
 EXTERN int		Tcl_RegExpMatch _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, char *pattern));
 EXTERN void		Tcl_RegExpRange _ANSI_ARGS_((Tcl_RegExp regexp,
@@ -1467,7 +1425,6 @@ EXTERN void		Tcl_RegisterChannel _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN void		Tcl_RegisterObjType _ANSI_ARGS_((
 			    Tcl_ObjType *typePtr));
 EXTERN void		Tcl_Release _ANSI_ARGS_((ClientData clientData));
-EXTERN void		Tcl_RestartIdleTimer _ANSI_ARGS_((void));
 EXTERN void		Tcl_ResetResult _ANSI_ARGS_((Tcl_Interp *interp));
 #define Tcl_Return Tcl_SetResult
 EXTERN int		Tcl_ScanCountedElement _ANSI_ARGS_((CONST char *string,
@@ -1508,9 +1465,6 @@ EXTERN void		Tcl_SetObjLength _ANSI_ARGS_((Tcl_Obj *objPtr,
 			    int length));
 EXTERN void		Tcl_SetObjResult _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *resultObjPtr));
-EXTERN Tcl_Obj *	Tcl_SetObjVar2 _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *part1, char *part2, Tcl_Obj *newValuePtr,
-			    int flags));
 EXTERN void		Tcl_SetPanicProc _ANSI_ARGS_((void (*proc)
 			    _ANSI_ARGS_(TCL_VARARGS(char *, format))));
 EXTERN int		Tcl_SetRecursionLimit _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1533,14 +1487,14 @@ EXTERN char *		Tcl_SignalMsg _ANSI_ARGS_((int sig));
 EXTERN void		Tcl_Sleep _ANSI_ARGS_((int ms));
 EXTERN void		Tcl_SourceRCFile _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		Tcl_SplitList _ANSI_ARGS_((Tcl_Interp *interp,
-			    CONST char *list, int *argcPtr, char ***argvPtr));
-EXTERN void		Tcl_SplitPath _ANSI_ARGS_((CONST char *path,
+			    char *list, int *argcPtr, char ***argvPtr));
+EXTERN void		Tcl_SplitPath _ANSI_ARGS_((char *path,
 			    int *argcPtr, char ***argvPtr));
 EXTERN void		Tcl_StaticPackage _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *pkgName, Tcl_PackageInitProc *initProc,
 			    Tcl_PackageInitProc *safeInitProc));
-EXTERN int		Tcl_StringMatch _ANSI_ARGS_((CONST char *string,
-			    CONST char *pattern));
+EXTERN int		Tcl_StringMatch _ANSI_ARGS_((char *string,
+			    char *pattern));
 EXTERN int		Tcl_Tell _ANSI_ARGS_((Tcl_Channel chan));
 #define Tcl_TildeSubst Tcl_TranslateFileName
 EXTERN int		Tcl_TraceVar _ANSI_ARGS_((Tcl_Interp *interp,
@@ -1589,20 +1543,10 @@ EXTERN int		Tcl_WaitForEvent _ANSI_ARGS_((Tcl_Time *timePtr));
 EXTERN Tcl_Pid		Tcl_WaitPid _ANSI_ARGS_((Tcl_Pid pid, int *statPtr, 
 			    int options));
 EXTERN int		Tcl_Write _ANSI_ARGS_((Tcl_Channel chan,
-			    CONST char *src, int srcLen));
+			    char *s, int slen));
 EXTERN void		Tcl_WrongNumArgs _ANSI_ARGS_((Tcl_Interp *interp,
 			    int objc, Tcl_Obj *CONST objv[], char *message));
 
-/* Andreas Kupries <a.kupries@westend.com>, 05/31/1997.
- * Support of Tcl-Trf.
- */
-EXTERN Tcl_Channel      Tcl_ReplaceChannel _ANSI_ARGS_ ((Tcl_Interp* interp,
-                          Tcl_ChannelType* typePtr, ClientData instanceData,
-                          int mask, Tcl_Channel prevChan));
-
-EXTERN void             Tcl_UndoReplaceChannel _ANSI_ARGS_ ((Tcl_Interp* interp,
-                          Tcl_Channel chan));
-
 #endif /* RESOURCE_INCLUDED */
 
 #undef TCL_STORAGE_CLASS
