@@ -4,15 +4,16 @@
  * Implements the Macintosh specific portions of the file manipulation
  * subcommands of the "file" command.
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1996-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclMacFCmd.c 1.11 96/10/10 10:13:49
+ * RCS: @(#) $Id: tclMacFCmd.c,v 1.2 1998/09/14 18:40:05 stanton Exp $
  */
 
 #include "tclInt.h"
+#include "tclMac.h"
 #include "tclMacInt.h"
 #include "tclPort.h"
 #include <FSpCompat.h>
@@ -23,35 +24,67 @@
 #include <DirectoryCopy.h>
 #include <Script.h>
 #include <string.h>
+#include <Finder.h>
+
+/*
+ * Callback for the file attributes code.
+ */
+
+static int		GetFileFinderAttributes _ANSI_ARGS_((Tcl_Interp *interp,
+			    int objIndex, char *fileName,
+			    Tcl_Obj **attributePtrPtr));
+static int		GetFileReadOnly _ANSI_ARGS_((Tcl_Interp *interp,
+			    int objIndex, char *fileName,
+			    Tcl_Obj **readOnlyPtrPtr));
+static int		SetFileFinderAttributes _ANSI_ARGS_((Tcl_Interp *interp,
+			    int objIndex, char *fileName,
+			    Tcl_Obj *attributePtr));
+static int		SetFileReadOnly _ANSI_ARGS_((Tcl_Interp *interp,
+			    int objIndex, char *fileName,
+			    Tcl_Obj *readOnlyPtr));
+
+/*
+ * These are indeces into the tclpFileAttrsStrings table below.
+ */
+
+#define MAC_CREATOR_ATTRIBUTE	0
+#define MAC_HIDDEN_ATTRIBUTE	1
+#define MAC_READONLY_ATTRIBUTE	2
+#define MAC_TYPE_ATTRIBUTE	3
+
+/*
+ * Global variables for the file attributes code.
+ */
+
+char *tclpFileAttrStrings[] = {"-creator", "-hidden", "-readonly",
+	"-type", (char *) NULL};
+CONST TclFileAttrProcs tclpFileAttrProcs[] = {
+	{GetFileFinderAttributes, SetFileFinderAttributes},
+	{GetFileFinderAttributes, SetFileFinderAttributes},
+	{GetFileReadOnly, SetFileReadOnly},
+	{GetFileFinderAttributes, SetFileFinderAttributes}};
+
 
 /*
  * Prototypes for procedure only used in this file
  */
 
-OSErr			FSpGetFLockCompat(const FSSpec *specPtr, 
-			    Boolean *lockedPtr);
-
-static OSErr		GenerateUniqueName(short vRefNum, long dirID1, 
-			    long dirID2, Str31 uniqueName);
-static OSErr		GetFileSpecs(char *path, FSSpec *pathSpecPtr,
+static pascal Boolean 	CopyErrHandler _ANSI_ARGS_((OSErr error, 
+			    short failedOperation,
+			    short srcVRefNum, long srcDirID,
+			    StringPtr srcName, short dstVRefNum,
+			    long dstDirID,StringPtr dstName));
+OSErr			FSpGetFLockCompat _ANSI_ARGS_((const FSSpec *specPtr, 
+			    Boolean *lockedPtr));
+static OSErr		GenerateUniqueName _ANSI_ARGS_((short vRefNum, 
+			    long dirID1, long dirID2, Str31 uniqueName));
+static OSErr		GetFileSpecs _ANSI_ARGS_((char *path, FSSpec *pathSpecPtr,
 			    FSSpec *dirSpecPtr,	Boolean *pathExistsPtr,	
-			    Boolean *pathIsDirectoryPtr);
-static OSErr		MoveRename(const FSSpec *srcSpecPtr, 
-			    const FSSpec *dstSpecPtr, StringPtr copyName);
-static int		Pstrequal(ConstStr255Param stringA, 
-			    ConstStr255Param stringB);
-
-static pascal Boolean CopyErrHandler(
-    OSErr error,              /* Error that occured */
-    short failedOperation,    /* operation that caused the error */
-    short srcVRefNum,         /* volume ref number of source */
-    long srcDirID,            /* directory id of source */
-    StringPtr srcName,        /* name of source */
-    short dstVRefNum,         /* volume ref number of dst */
-    long dstDirID,            /* directory id of dst */
-    StringPtr dstName         /* name of dst directory */
-) ;
-          
+			    Boolean *pathIsDirectoryPtr));
+static OSErr		MoveRename _ANSI_ARGS_((const FSSpec *srcSpecPtr, 
+			    const FSSpec *dstSpecPtr, StringPtr copyName));
+static int		Pstrequal _ANSI_ARGS_((ConstStr255Param stringA, 
+			    ConstStr255Param stringB));
                  
 /*
  *---------------------------------------------------------------------------
@@ -564,17 +597,18 @@ TclpCopyDirectory(
  *      
  *----------------------------------------------------------------------
  */
+
 static pascal Boolean 
 CopyErrHandler(
-    OSErr error,              /* Error that occured */
-    short failedOperation,    /* operation that caused the error */
-    short srcVRefNum,         /* volume ref number of source */
-    long srcDirID,            /* directory id of source */
-    StringPtr srcName,        /* name of source */
-    short dstVRefNum,         /* volume ref number of dst */
-    long dstDirID,            /* directory id of dst */
-    StringPtr dstName         /* name of dst directory */
-) {
+    OSErr error,		/* Error that occured */
+    short failedOperation,	/* operation that caused the error */
+    short srcVRefNum,		/* volume ref number of source */
+    long srcDirID,		/* directory id of source */
+    StringPtr srcName,		/* name of source */
+    short dstVRefNum,		/* volume ref number of dst */
+    long dstDirID,		/* directory id of dst */
+    StringPtr dstName)		/* name of dst directory */
+{
     return true;
 }
 
@@ -995,10 +1029,11 @@ FSpGetFLockCompat(
  *      
  *----------------------------------------------------------------------
  */
+
 static int 
 Pstrequal (
     ConstStr255Param stringA,	/* Pascal string A */
-    ConstStr255Param stringB)    /* Pascal string B */
+    ConstStr255Param stringB)   /* Pascal string B */
 {
     int i, len;
     
@@ -1010,5 +1045,364 @@ Pstrequal (
     }
     return 1;
 }
+    
+/*
+ *----------------------------------------------------------------------
+ *
+ * GetFileFinderAttributes --
+ *
+ *	Returns a Tcl_Obj containing the value of a file attribute
+ *	which is part of the FInfo record. Which attribute is controlled
+ *	by objIndex.
+ *
+ * Results:
+ *      Returns a standard TCL error. If the return value is TCL_OK,
+ *	the new creator or file type object is put into attributePtrPtr.
+ *	The object will have ref count 0. If there is an error,
+ *	attributePtrPtr is not touched.
+ *
+ * Side effects:
+ *      A new object is allocated if the file is valid.
+ *      
+ *----------------------------------------------------------------------
+ */
 
+static int
+GetFileFinderAttributes(
+    Tcl_Interp *interp,		/* The interp to report errors with. */
+    int objIndex,		/* The index of the attribute option. */
+    char *fileName,		/* The name of the file. */
+    Tcl_Obj **attributePtrPtr)	/* A pointer to return the object with. */
+{
+    OSErr err;
+    FSSpec fileSpec;
+    FInfo finfo;
+    
+    err = FSpLocationFromPath(strlen(fileName), fileName, &fileSpec);
+    
+    if (err == noErr) {
+    	err = FSpGetFInfo(&fileSpec, &finfo);
+    }
+    
+    if (err == noErr) {
+    	switch (objIndex) {
+    	    case MAC_CREATOR_ATTRIBUTE:
+    	    	*attributePtrPtr = Tcl_NewOSTypeObj(finfo.fdCreator);
+    	    	break;
+    	    case MAC_HIDDEN_ATTRIBUTE:
+    	    	*attributePtrPtr = Tcl_NewBooleanObj(finfo.fdFlags
+    	    		& kIsInvisible);
+    	    	break;
+    	    case MAC_TYPE_ATTRIBUTE:
+    	    	*attributePtrPtr = Tcl_NewOSTypeObj(finfo.fdType);
+    	    	break;
+    	}
+    } else if (err == fnfErr) {
+    	long dirID;
+    	Boolean isDirectory = 0;
+    	
+    	err = FSpGetDirectoryID(&fileSpec, &dirID, &isDirectory);
+    	if ((err == noErr) && isDirectory) {
+    	    if (objIndex == MAC_HIDDEN_ATTRIBUTE) {
+    	    	*attributePtrPtr = Tcl_NewBooleanObj(0);
+    	    } else {
+    	    	*attributePtrPtr = Tcl_NewOSTypeObj('Fldr');
+    	    }
+    	}
+    }
+    
+    if (err != noErr) {
+    	errno = TclMacOSErrorToPosixError(err);
+    	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), 
+    		"couldn't get attributes for file \"", fileName, "\": ",
+    		Tcl_PosixError(interp), (char *) NULL);
+    	return TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * GetFileReadOnly --
+ *
+ *	Returns a Tcl_Obj containing a Boolean value indicating whether
+ *	or not the file is read-only. The object will have ref count 0.
+ *	This procedure just checks the Finder attributes; it does not
+ *	check AppleShare sharing attributes.
+ *
+ * Results:
+ *      Returns a standard TCL error. If the return value is TCL_OK,
+ *	the new creator type object is put into readOnlyPtrPtr.
+ *	If there is an error, readOnlyPtrPtr is not touched.
+ *
+ * Side effects:
+ *      A new object is allocated if the file is valid.
+ *      
+ *----------------------------------------------------------------------
+ */
+
+static int
+GetFileReadOnly(
+    Tcl_Interp *interp,		/* The interp to report errors with. */
+    int objIndex,		/* The index of the attribute. */
+    char *fileName,		/* The name of the file. */
+    Tcl_Obj **readOnlyPtrPtr)	/* A pointer to return the object with. */
+{
+    OSErr err;
+    FSSpec fileSpec;
+    CInfoPBRec paramBlock;
+    
+    err = FSpLocationFromPath(strlen(fileName), fileName, &fileSpec);
+    
+    if (err == noErr) {
+    	if (err == noErr) {
+    	    paramBlock.hFileInfo.ioCompletion = NULL;
+    	    paramBlock.hFileInfo.ioNamePtr = fileSpec.name;
+    	    paramBlock.hFileInfo.ioVRefNum = fileSpec.vRefNum;
+    	    paramBlock.hFileInfo.ioFDirIndex = 0;
+    	    paramBlock.hFileInfo.ioDirID = fileSpec.parID;
+    	    err = PBGetCatInfo(&paramBlock, 0);
+    	    if (err == noErr) {
+    	    
+    	    	/*
+    	    	 * For some unknown reason, the Mac does not give
+    	    	 * symbols for the bits in the ioFlAttrib field.
+    	    	 * 1 -> locked.
+    	    	 */
+    	    
+    	    	*readOnlyPtrPtr = Tcl_NewBooleanObj(
+    	    		paramBlock.hFileInfo.ioFlAttrib & 1);
+    	    }
+    	}
+    }
+    if (err != noErr) {
+    	errno = TclMacOSErrorToPosixError(err);
+    	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), 
+    		"couldn't get attributes for file \"", fileName, "\": ",
+    		Tcl_PosixError(interp), (char *) NULL);
+    	return TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * SetFileFinderAttributes --
+ *
+ *	Sets the file to the creator or file type given by attributePtr.
+ *	objIndex determines whether the creator or file type is set.
+ *
+ * Results:
+ *	Returns a standard TCL error.
+ *
+ * Side effects:
+ *      The file's attribute is set.
+ *      
+ *----------------------------------------------------------------------
+ */
+
+static int
+SetFileFinderAttributes(
+    Tcl_Interp *interp,		/* The interp to report errors with. */
+    int objIndex,		/* The index of the attribute. */
+    char *fileName,		/* The name of the file. */
+    Tcl_Obj *attributePtr)	/* The command line object. */
+{
+    OSErr err;
+    FSSpec fileSpec;
+    FInfo finfo;
+    
+    err = FSpLocationFromPath(strlen(fileName), fileName, &fileSpec);
+    
+    if (err == noErr) {
+    	err = FSpGetFInfo(&fileSpec, &finfo);
+    }
+    
+    if (err == noErr) {
+    	switch (objIndex) {
+    	    case MAC_CREATOR_ATTRIBUTE:
+    	    	if (Tcl_GetOSTypeFromObj(interp, attributePtr,
+    	    		&finfo.fdCreator) != TCL_OK) {
+    	    	    return TCL_ERROR;
+    	    	}
+    	    	break;
+    	    case MAC_HIDDEN_ATTRIBUTE: {
+    	    	int hidden;
+    	    	
+    	    	if (Tcl_GetBooleanFromObj(interp, attributePtr, &hidden)
+    	    		!= TCL_OK) {
+    	    	    return TCL_ERROR;
+    	    	}
+    	    	if (hidden) {
+    	    	    finfo.fdFlags |= kIsInvisible;
+    	    	} else {
+    	    	    finfo.fdFlags &= ~kIsInvisible;
+    	    	}
+    	    	break;
+    	    }
+    	    case MAC_TYPE_ATTRIBUTE:
+    	    	if (Tcl_GetOSTypeFromObj(interp, attributePtr,
+    	    		&finfo.fdType) != TCL_OK) {
+    	    	    return TCL_ERROR;
+    	    	}
+    	    	break;
+    	}
+    	err = FSpSetFInfo(&fileSpec, &finfo);
+    } else if (err == fnfErr) {
+    	long dirID;
+    	Boolean isDirectory = 0;
+    	
+    	err = FSpGetDirectoryID(&fileSpec, &dirID, &isDirectory);
+    	if ((err == noErr) && isDirectory) {
+    	    Tcl_Obj *resultPtr = Tcl_GetObjResult(interp);
+    	    Tcl_AppendStringsToObj(resultPtr, "cannot set ",
+    	    	    tclpFileAttrStrings[objIndex], ": \"",
+    	    	    fileName, "\" is a directory", (char *) NULL);
+    	    return TCL_ERROR;
+    	}
+    }
+    
+    if (err != noErr) {
+    	errno = TclMacOSErrorToPosixError(err);
+    	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), 
+    		"couldn't set attributes for file \"", fileName, "\": ",
+    		Tcl_PosixError(interp), (char *) NULL);
+    	return TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * SetFileReadOnly --
+ *
+ *	Sets the file to be read-only according to the Boolean value
+ *	given by hiddenPtr.
+ *
+ * Results:
+ *	Returns a standard TCL error.
+ *
+ * Side effects:
+ *      The file's attribute is set.
+ *      
+ *----------------------------------------------------------------------
+ */
+
+static int
+SetFileReadOnly(
+    Tcl_Interp *interp,		/* The interp to report errors with. */
+    int objIndex,		/* The index of the attribute. */
+    char *fileName,		/* The name of the file. */
+    Tcl_Obj *readOnlyPtr)	/* The command line object. */
+{
+    OSErr err;
+    FSSpec fileSpec;
+    HParamBlockRec paramBlock;
+    int hidden;
+    
+    err = FSpLocationFromPath(strlen(fileName), fileName, &fileSpec);
+    
+    if (err == noErr) {
+    	if (Tcl_GetBooleanFromObj(interp, readOnlyPtr, &hidden) != TCL_OK) {
+    	    return TCL_ERROR;
+    	}
+    
+    	paramBlock.fileParam.ioCompletion = NULL;
+    	paramBlock.fileParam.ioNamePtr = fileSpec.name;
+    	paramBlock.fileParam.ioVRefNum = fileSpec.vRefNum;
+    	paramBlock.fileParam.ioDirID = fileSpec.parID;
+    	if (hidden) {
+    	    err = PBHSetFLock(&paramBlock, 0);
+    	} else {
+    	    err = PBHRstFLock(&paramBlock, 0);
+    	}
+    }
+    
+    if (err == fnfErr) {
+    	long dirID;
+    	Boolean isDirectory = 0;
+    	err = FSpGetDirectoryID(&fileSpec, &dirID, &isDirectory);
+    	if ((err == noErr) && isDirectory) {
+    	    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+    	    	    "cannot set a directory to read-only when File Sharing is turned off",
+    	    	    (char *) NULL);
+    	    return TCL_ERROR;
+    	} else {
+    	    err = fnfErr;
+    	}
+    }
+    
+    if (err != noErr) {
+    	errno = TclMacOSErrorToPosixError(err);
+    	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), 
+    		"couldn't set attributes for file \"", fileName, "\": ",
+    		Tcl_PosixError(interp), (char *) NULL);
+    	return TCL_ERROR;
+    }
+    return TCL_OK;
+}
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * TclpListVolumes --
+ *
+ *	Lists the currently mounted volumes
+ *
+ * Results:
+ *	A standard Tcl result.  Will always be TCL_OK, since there is no way
+ *	that this command can fail.  Also, the interpreter's result is set to 
+ *	the list of volumes.
+ *
+ * Side effects:
+ *	None
+ *
+ *---------------------------------------------------------------------------
+ */
+
+int
+TclpListVolumes( 
+		Tcl_Interp *interp)    /* Interpreter to which to pass the volume list */
+{
+    HParamBlockRec pb;
+    Str255 name;
+    OSErr theError = noErr;
+    Tcl_Obj *resultPtr, *elemPtr;
+    short volIndex = 1;
+
+    resultPtr = Tcl_NewObj();
+        
+    /*
+     * We use two facts:
+     * 1) The Mac volumes are enumerated by the ioVolIndex parameter of
+     * the HParamBlockRec.  They run through the integers contiguously, 
+     * starting at 1.  
+     * 2) PBHGetVInfoSync returns an error when you ask for a volume index
+     * that does not exist.
+     * 
+     */
+        
+    while ( 1 ) {
+        pb.volumeParam.ioNamePtr = (StringPtr) & name;
+        pb.volumeParam.ioVolIndex = volIndex;
+                
+        theError = PBHGetVInfoSync(&pb);
+
+        if ( theError != noErr ) {
+            break;
+        }
+                
+        elemPtr = Tcl_NewStringObj((char *) name + 1, (int) name[0]);
+        Tcl_AppendToObj(elemPtr, ":", 1);
+        Tcl_ListObjAppendElement(interp, resultPtr, elemPtr);
+                
+        volIndex++;             
+    }
+        
+    Tcl_SetObjResult(interp, resultPtr);
+    return TCL_OK;      
+}
 
